/***************************************************************************
                          dockwidget.cpp  -  description
                             -------------------
    begin                : Thu Jul 6 2000
    copyright            : (C) 2000 by Ian Reinhart Geiser
    email                : geiseri@msoe.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dockwidget.h"

#include <qtooltip.h>
#include <qimage.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qpushbutton.h>

#include <klocale.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kglobalsettings.h>

dockwidget::dockwidget(QWidget *parent, const char *name) : QWidget(parent,name){
    initDock();
    connect(m_button, SIGNAL( released() ), SLOT( buttonHandler() ));

    m_label1->setText(i18n("Unavailable"));
    m_label2->setText(i18n("data"));
    m_label3->setText("...");
}

dockwidget::~dockwidget(){
}

void dockwidget::setViewMode(int _mode)
{
	kdDebug(12004) << "View mode " << _mode << endl;
    mode = _mode;
}

void dockwidget::setWeatherIcon(const QPixmap &icon)
{
    m_icon = icon;
}

/** Change the button icon */
void dockwidget::showIcon(){
    QSize margin( 6, 6 );

    QImage image = m_icon.convertToImage();
    image = image.smoothScale( m_button->size()-margin, QImage::ScaleMin );

    m_button->setPixmap( QPixmap( image ) );
}

void dockwidget::showWeather()
{
    QString tip;

    if (mode == 1)
    {
        tip = i18n("Temperature: ") + m_temperature;
        tip += i18n("\nWind: ") + m_wind;
        tip += i18n("\nAir Pressure: ") + m_pressure;
    }
    else
    {
        tip = i18n("Click here to see\nthe weather report...");
    }

    QToolTip::remove(m_button);
    QToolTip::add(m_button, tip);

    showIcon();

    m_label1->setText(m_temperature);
    m_label2->setText(m_wind);
    m_label3->setText(m_pressure);

    if (mode == 1)
    {
	m_label1->hide();
	m_label2->hide();
	m_label3->hide();
    }
    else if (mode == 2)
    {
	m_label1->show();
	m_label2->hide();
	m_label3->hide();
    }
    else if (mode == 3)
    {
	m_label1->show();
	m_label2->show();
	m_label3->show();
    }
    updateFont();
}

/** relay the button pressed to the parent widget */
void dockwidget::buttonHandler(){
    emit buttonEvent();
}

void dockwidget::initDock(){
    kdDebug(12004) << "Init Dialog " << endl;

    m_button= new QPushButton(this,"NoName");

    m_icon = QPixmap( locate( "data", "kweather/dunno.png" ) );
    showIcon();

    m_label1= new QLabel(this,"NoName");
    m_label2= new QLabel(this,"NoName");
    m_label3= new QLabel(this,"NoName");

    m_label1->setTextFormat(Qt::PlainText);
    m_label2->setTextFormat(Qt::PlainText);
    m_label3->setTextFormat(Qt::PlainText);

    m_label1->setMargin(0);
    m_label2->setMargin(0);
    m_label3->setMargin(0);

    updateFont();
}

/** Set the current temperature. */
void dockwidget::setTemperature( QString s){
    m_temperature = s;
}

/** Set the current wind speed and direction data. */
void dockwidget::setWind( QString s){
    m_wind = s;
}

/** Set the current pressure. */
void dockwidget::setPressure( QString s){
    m_pressure = s;
}

/** resize the view **/
void dockwidget::resizeView(int w, int h)
{
    kdDebug(12004) << "Changing to size " << w << "," << h << endl;
    resize(w, h);
    if (w >= h)
    {
        // Kicker in horizontal mode
	if( mode == 3)
	{
		m_label1->setGeometry(h+2, 0, w-h, h/3);
		m_label2->setGeometry(h+2, h/3, w-h, h/3);
		m_label3->setGeometry(h+2, 2*h/3, w-h, h/3);
	}
	else if ( mode == 2)
	{
		m_label1->setGeometry(h+2, 0, w-h, h);
	}
	m_button->setGeometry(0,0,h,h);

    }
    else
    {
        // Kicker in vertical mode
if( mode == 3)
	{
		m_label1->setGeometry(2, w, w, w/3);
		m_label2->setGeometry(2, 4*w/3, w, w/3);
		m_label3->setGeometry(2, 5*w/3, w, w/3);
	}
	else if ( mode == 2)
	{
		m_label1->setGeometry(2, w, w, w);
	}
	m_button->setGeometry(0,0,w,w);
    }
    updateFont();
    showIcon();
}

QSize dockwidget::sizeHint(int w, int h)
{
	int x = w;
	int y = h;

	QFont font = KGlobalSettings::generalFont();


	if (w >= h)
	{
		// Kicker in horizontal mode
		if( mode == 3)
		{
			font.setPixelSize( h/3-1 );
			QFontMetrics fm(font);
			x = h + QMAX(fm.width(m_label2->text()),
				fm.width(m_label3->text())) + 2;
			kdDebug(12004) << "text: " << m_label2->text() << "old width: " << w
				<< " new width: " << x << endl;
			return QSize(x,y);
		}
		else if ( mode == 2)
		{
			font.setPixelSize( h );
			QFontMetrics fm(font);
			x = h + fm.width(m_label1->text());
			kdDebug(12004) << "text: " << m_label1->text() << " old width: " << w
				<< " new width: " << x << endl;
			return QSize(x,y);
		}
		else
			return QSize(y,y);
	}
	else
	{
		return QSize(x,y);
	}
}

void dockwidget::updateFont()
{
    kdDebug(12004) << "Update font: " << m_button->height() << endl;
    QFont font = KGlobalSettings::generalFont();
    font.setPixelSize( m_label1->height()-1 );
    m_label1->setFont(font);
    m_label2->setFont(font);
    m_label3->setFont(font);
}

void dockwidget::resizeEvent ( QResizeEvent *e )
{
	int width = e->size().width();
	int height = e->size().height();
	kdDebug(12004) << "Handle resize event..." << endl;
	if( width < 90)
		width = 90;
	if( height < 48)
		height = 48;

	resizeView(width, height );
}

#include "dockwidget.moc"

// vim:ts=4:sw=4:et
